﻿using C1.WPF.Chart;
using C1.Chart;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices.JavaScript;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
//using System.Drawing;
using System.Windows.Media.Media3D;
using System.Diagnostics;
using System.Windows.Media;
using System.Globalization;

namespace GanttChartDemo
{
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    public static List<Task> taskData;

    public MainWindow()
    {
      InitializeComponent();

      taskData = GetTasksData();
      flexchart.ChartType = ChartType.Bar;
      flexchart.BindingX = "Name";
      flexchart.ItemsSource = taskData;
      flexchart.Binding = "Start,End";

      // tooltip contains point index (as string) that will be used in binding converter
      flexchart.Series.Add(new GanntSeries(taskData) { Tooltip = "{pointIndex}" });

      flexchart.DataLabel.Content = "{DurationInWeeks:n1} Weeks";
      flexchart.DataLabel.Position = LabelPosition.Left;
      flexchart.AxisX.Min = taskData[0].Start.ToOADate();
      flexchart.AxisX.MajorUnit = 7D;
      flexchart.AxisY.Reversed = true;
    }

    public static List<Task> GetTasksData()
    {
      var taskList = new List<Task>();
      var year = DateTime.Now.Year;
      taskList.Add(new Task { Name = "Plan", Start = new DateTime(year, 5, 5), End = new DateTime(year, 5, 11) });
      taskList.Add(new Task { Name = "Design", Start = new DateTime(year, 5, 12), End = new DateTime(year, 5, 18) });
      taskList.Add(new Task { Name = "Development", Start = new DateTime(year, 5, 19), End = new DateTime(year, 6, 30) });
      taskList.Add(new Task { Name = "Test Accept", Start = new DateTime(year, 7, 1), End = new DateTime(year, 7, 14) });
      taskList.Add(new Task { Name = "Distribution", Start = new DateTime(year, 7, 14), End = new DateTime(year, 7, 21) });

      return taskList;
    }
  }

  public class GanntSeries : Series, ISeries
  {
    List<Task> taskData;
    public GanntSeries(List<Task> taskData)
    {
      this.taskData = taskData;
      SymbolRendering += (s, e) =>
      {
        // next color in palette
        var color = ((IPalette)ActualChart).GetColor(e.Index);
        e.Engine.SetFill(color);
        e.Engine.SetStroke(color);
      };
    }

    // multiple legend items for the series
    string ISeries.GetLegendItemName(int index) => taskData[index].Name;

    _Style ISeries.GetLegendItemStyle(int index) => new _Style { Fill = ((IPalette)ActualChart).GetColor(index) };

    int ISeries.GetLegendItemLength() => taskData.Count;
  }

  public class Task
  {
    public string Name { get; set; } // The name of the task
    public DateTime Start { get; set; } // The start date of the task
    public DateTime End { get; set; } // The end date of the task

    // Optional: You can add a method to calculate the duration of the task
    public TimeSpan Duration => (End - Start);

    public double DurationInWeeks => Duration.TotalDays / 7;
  }

  public class TooltipConverter : IMultiValueConverter
  {
    public object? Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
    {
      // get current task from chart's itemsSource and pointIndex
      Task? task = null;
      if (values?.Length == 2)
      {
        var chart = values[0] as FlexChart;
        var s = values[1]?.ToString();

        if (chart != null && int.TryParse(s, out int pointIndex))
        {
          var tasks = chart.ItemsSource as List<Task>;

          if (tasks != null && pointIndex >= 0 && pointIndex < tasks.Count)
            task = tasks[pointIndex];
        }
      }

      return task;
    }

    public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
    {
      throw new NotImplementedException();
    }
  }
}